@@ -27,7 +27,7 @@
 *
 ********************************************************************************
 *
-*  $Id: psad.h 1536 2006-05-02 03:06:18Z mbr $
+*  $Id: psad.h 1783 2006-12-09 00:39:48Z mbr $
 */
 
 #ifndef __PSAD_H__
@@ -44,6 +44,7 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <syslog.h>
+#include <ctype.h>
 
 /* DEFINES ********************************************************************/
 #define MAX_LINE_BUF 1024
@@ -51,6 +52,7 @@
 #define MAX_PATH_LEN 100
 #define MAX_MSG_LEN 120
 #define MAX_GEN_LEN 80
+#define MAX_EMAIL_LEN 300
 #define MAX_ARG_LEN 30
 #define MAX_NUM_LEN 6
 
@@ -60,6 +62,10 @@ void check_unique_pid(const char *, const char *);
 void write_pid(const char *, pid_t);
 void daemonize_process(const char *);
 void send_alert_email(const char *, const char *, const char *);
+int has_sub_var(char *var_name, char *value, char *sub_var,
+    char *pre_str, char *post_str);
+void expand_sub_var_value(char *value, const char *sub_var,
+    const char *pre_str, const char *post_str);
 int find_char_var(char *, char *, char *);
 int check_import_config(time_t *config_mtime, char *config_file);
 
